package sun.nio.ch;

import java.nio.channels.AsynchronousCloseException;
import java.nio.ByteBuffer;
import java.io.IOException;
import java.nio.channels.spi.SelectorProvider;
import java.io.FileDescriptor;
import java.nio.channels.SocketChannel;
import java.nio.channels.Pipe;

class SourceChannelImpl extends Pipe.SourceChannel implements SelChImpl
{
    SocketChannel sc;
    
    @Override
    public FileDescriptor getFD() {
        return ((SocketChannelImpl)this.sc).getFD();
    }
    
    @Override
    public int getFDVal() {
        return ((SocketChannelImpl)this.sc).getFDVal();
    }
    
    SourceChannelImpl(SelectorProvider provider, SocketChannel sc) {
        super(provider);
        this.sc = sc;
    }
    
    @Override
    protected void implCloseSelectableChannel() throws IOException {
        if (!this.isRegistered()) {
            this.kill();
        }
    }
    
    @Override
    public void kill() throws IOException {
        this.sc.close();
    }
    
    @Override
    protected void implConfigureBlocking(boolean block) throws IOException {
        this.sc.configureBlocking(block);
    }
    
    public boolean translateReadyOps(int n, int n2, SelectionKeyImpl selectionKeyImpl) {
        int nioInterestOps = selectionKeyImpl.nioInterestOps();
        int nioReadyOps = selectionKeyImpl.nioReadyOps();
        if ((n & Net.POLLNVAL) != 0x0) {
            throw new Error("POLLNVAL detected");
        }
        if ((n & (Net.POLLERR | Net.POLLHUP)) != 0x0) {
            selectionKeyImpl.nioReadyOps(nioInterestOps);
            return (nioInterestOps & ~nioReadyOps) != 0x0;
        }
        if ((n & Net.POLLIN) != 0x0 && (nioInterestOps & 0x1) != 0x0) {
            n2 |= 0x1;
        }
        selectionKeyImpl.nioReadyOps(n2);
        return (n2 & ~nioReadyOps) != 0x0;
    }
    
    @Override
    public boolean translateAndUpdateReadyOps(int n, SelectionKeyImpl selectionKeyImpl) {
        return this.translateReadyOps(n, selectionKeyImpl.nioReadyOps(), selectionKeyImpl);
    }
    
    @Override
    public boolean translateAndSetReadyOps(int n, SelectionKeyImpl selectionKeyImpl) {
        return this.translateReadyOps(n, 0, selectionKeyImpl);
    }
    
    @Override
    public void translateAndSetInterestOps(int pollin, SelectionKeyImpl selectionKeyImpl) {
        if ((pollin & 0x1) != 0x0) {
            pollin = Net.POLLIN;
        }
        selectionKeyImpl.selector.putEventOps(selectionKeyImpl, pollin);
    }
    
    @Override
    public int read(ByteBuffer byteBuffer) throws IOException {
        try {
            return this.sc.read(byteBuffer);
        }
        catch (AsynchronousCloseException ex) {
            this.close();
            throw ex;
        }
    }
    
    @Override
    public long read(ByteBuffer[] array, int n, int n2) throws IOException {
        if (n < 0 || n2 < 0 || n > array.length - n2) {
            throw new IndexOutOfBoundsException();
        }
        try {
            return this.read(Util.subsequence(array, n, n2));
        }
        catch (AsynchronousCloseException ex) {
            this.close();
            throw ex;
        }
    }
    
    @Override
    public long read(ByteBuffer[] dsts) throws IOException {
        try {
            return this.sc.read(dsts);
        }
        catch (AsynchronousCloseException ex) {
            this.close();
            throw ex;
        }
    }
}
